@@ -35,7 +35,7 @@ import os
 import pickle
 import re
 import sys
-import unittest2 as unittest
+import unittest
 import urllib
 
 from parameterized import parameterized
